Attribute VB_Name = "mDevInterface"
Option Explicit

Public Enum DEVICE_TYPE
    DBT_DEVTYP_UNKNOWN = -1&
    DBT_DEVTYP_OEM = 0&
    DBT_DEVTYP_DEVNODE = 1&
    DBT_DEVTYP_VOLUME = 2&
    DBT_DEVTYP_PORT = 3&
    DBT_DEVTYP_NET = 4&
    DBT_DEVTYP_DEVICEINTERFACE = 5&
    DBT_DEVTYP_HANDLE = 6&
End Enum

Public Enum DEVICE_EVENTS
    DBT_DEVNODES_CHANGED = 7&
    DBT_QUERYCHANGECONFIG = 23&
    DBT_CONFIGCHANGED = 24&
    DBT_CONFIGCHANGECANCELED = 25&
    DBT_CUSTOMEVENT = 32744&
    DBT_DEVICEARRIVAL = 32768
    DBT_DEVICEQUERYREMOVE = 32769
    DBT_DEVICEQUERYREMOVEFAILED = 32770
    DBT_DEVICEREMOVEPENDING = 32771
    DBT_DEVICEREMOVECOMPLETE = 32772
    DBT_DEVICETYPESPECIFIC = 32773
    DBT_USERDEFINED = 65535
End Enum

Public Type GUID
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(1 To 8) As Byte
End Type

Public Type DEV_BROADCAST_HDR
   cbSize As Long
   devType As Long
   Reserved As Long
End Type

Public Type DEV_BROADCAST_DEVICEINTERFACE
   hdr As DEV_BROADCAST_HDR
   ClassGuid As GUID
   Name As Long
End Type

Public Type DEV_BROADCAST_DEVICEINTERFACE2
   hdr As DEV_BROADCAST_HDR
   ClassGuid As GUID
   Name As String * 1024
End Type

Public Type DEV_BROADCAST_VOLUME
   hdr As DEV_BROADCAST_HDR
   UnitMask As Long
   flags As Integer
End Type
Public Const DBTF_MEDIA As Long = &H1
Public Const DBTF_NET As Long = &H2

Public Declare Function RegisterDeviceNotification Lib "user32.dll" Alias "RegisterDeviceNotificationA" ( _
            ByVal hRecipient As Long, NotificationFilter As Any, ByVal flags As Long) As Long
            
Public Declare Function UnregisterDeviceNotification Lib "user32.dll" (ByVal hRecipient As Long) As Long
            
Public Const DEVICE_NOTIFY_ALL_INTERFACE_CLASSES = &H4
Public Const DEVICE_NOTIFY_WINDOW_HANDLE = 0
Public Const UNSAFE_REMOVE = &H1C

Public Declare Function IIDFromString Lib "ole32" (ByVal lpsz As Long, ByRef lpiid As GUID) As Long
Public Declare Function StringFromGUID2 Lib "ole32" (rguid As GUID, ByVal lpsz As Long, ByVal cchMax As Long) As Long

Public Const GUID_DEVINTERFACE_USB_DEVICE   As String = "{A5DCBF10-6530-11D2-901F-00C04FB951ED}"
Public Const GUID_DEVINTERFACE_USB_HUB      As String = "{F18A0E88-C30C-11D0-8815-00A0C906BED8}"
Public Const GUID_DEVINTERFACE_DISK         As String = "{53F56307-B6BF-11D0-94F2-00A0C91EFB8B}"
Public Const GUID_DEVINTERFACE_CDROM        As String = "{53F56308-B6BF-11D0-94F2-00A0C91EFB8B}"
Public Const GUID_DEVINTERFACE_PARTITION    As String = "{53F5630A-B6BF-11D0-94F2-00A0C91EFB8B}"
Public Const GUID_DEVINTERFACE_WRITEONCEDISK As String = "{53F5630C-B6BF-11D0-94F2-00A0C91EFB8B}"
Public Const GUID_DEVINTERFACE_VOLUME       As String = "{53F5630D-B6BF-11D0-94F2-00A0C91EFB8B}"
Public Const GUID_DEVINTERFACE_FLOPPY       As String = "{53F56311-B6BF-11D0-94F2-00A0C91EFB8B}"
Public Const GUID_DEVINTERFACE_CDCHANGER    As String = "{53F56312-B6BF-11D0-94F2-00A0C91EFB8B}"

Public Function RegisterDevNotify(ByVal hWnd As Long, ByRef hDevNotify As Long) As Long
    Dim flt As DEV_BROADCAST_DEVICEINTERFACE
        
    flt.hdr.cbSize = Len(flt)
    flt.hdr.devType = DBT_DEVTYP_DEVICEINTERFACE
    hDevNotify = RegisterDeviceNotification(hWnd, flt, DEVICE_NOTIFY_WINDOW_HANDLE Or DEVICE_NOTIFY_ALL_INTERFACE_CLASSES)
    If hDevNotify = 0 Then RegisterDevNotify = Err.LastDllError
End Function

Public Function GetDeviceType(devType As DEVICE_TYPE) As String
    Dim s As String
    Select Case devType
           Case DBT_DEVTYP_OEM
                s = "DBT_DEVTYP_OEM"
           Case DBT_DEVTYP_DEVNODE
                s = "DBT_DEVTYP_DEVNODE"
           Case DBT_DEVTYP_VOLUME
                s = "DBT_DEVTYP_VOLUME"
           Case DBT_DEVTYP_PORT
                s = "DBT_DEVTYP_PORT"
           Case DBT_DEVTYP_NET
                s = "DBT_DEVTYP_NET"
           Case DBT_DEVTYP_DEVICEINTERFACE
                s = "DBT_DEVTYP_DEVICEINTERFACE"
           Case DBT_DEVTYP_HANDLE
                s = "DBT_DEVTYP_HANDLE"
           Case Else
              s = "Device Type unknown: " & CStr(devType)
      End Select
      GetDeviceType = s
End Function

Public Function GetDeviceEvent(evt As DEVICE_EVENTS) As String
    Dim s As String
    Select Case evt
           Case DBT_CONFIGCHANGED
                s = "DBT_CONFIGCHANGED"
           Case DBT_CONFIGCHANGECANCELED
                s = "DBT_CONFIGCHANGECANCELED"
           Case DBT_CUSTOMEVENT
                s = "DBT_CUSTOMEVENT"
           Case DBT_DEVICEARRIVAL
                s = "DBT_DEVICEARRIVAL"
           Case DBT_DEVICEQUERYREMOVE
                s = "DEVICEQUERYREMOVE"
           Case DBT_DEVICEQUERYREMOVEFAILED
                s = "DEVICEQUERYREMOVEFAILED"
           Case DBT_DEVICEREMOVECOMPLETE
                s = "DBT_DEVICEREMOVECOMPLETE"
           Case DBT_DEVICEREMOVEPENDING
                s = "DEVICEREMOVEPENDING"
           Case DBT_DEVICETYPESPECIFIC
                s = "DEVICETYPESPECIFIC"
           Case DBT_DEVNODES_CHANGED
                s = "DEVNODES_CHANGED"
           Case DBT_QUERYCHANGECONFIG
                s = "QUERYCHANGECONFIG"
           Case DBT_USERDEFINED
                s = "USERDEFINED"
           Case Else
                s = "Unknown event = " & evt
      End Select
      GetDeviceEvent = s
End Function

Public Function IFaceFromGuid(ByVal sGUID As String) As String
    Dim s As String
    Select Case UCase(sGUID)
        Case GUID_DEVINTERFACE_USB_DEVICE:  s = "USB_DEVICE"
        Case GUID_DEVINTERFACE_USB_HUB:     s = "USB_HUB"
        Case GUID_DEVINTERFACE_DISK:        s = "DISK"
        Case GUID_DEVINTERFACE_CDROM:       s = "CDROM"
        Case GUID_DEVINTERFACE_PARTITION:   s = "PARTITION"
        Case GUID_DEVINTERFACE_WRITEONCEDISK: s = "WRITEONCEDISK"
        Case GUID_DEVINTERFACE_VOLUME:      s = "VOLUME"
        Case GUID_DEVINTERFACE_FLOPPY:      s = "FLOPPY"
        Case GUID_DEVINTERFACE_CDCHANGER:   s = "CDCHANGER"
        Case Else: s = "Unrecognized"
    End Select
    IFaceFromGuid = s
End Function

Public Function VolumeFromMask(ByVal UnitMask As Long) As String
    Dim i As Long
    For i = 0 To 25
        If (UnitMask And 2 ^ i) <> 0 Then
            VolumeFromMask = Chr$(65 + i)
            Exit For
        End If
    Next i
End Function

Public Function StringFromGUID(iid As GUID) As String
   Dim s As String
   s = Space(40)
   StringFromGUID2 iid, StrPtr(s), Len(s)
   StringFromGUID = Trim(TrimNull(s))
End Function
