Attribute VB_Name = "mRegistry"
Option Explicit

Public Enum RegValueTypes
    REG_NONE = 0&
    REG_SZ = 1&
    REG_EXPAND_SZ = 2&
    REG_BINARY = 3&
    REG_DWORD = 4&
    REG_DWORD_LITTLE_ENDIAN = 4&
    REG_DWORD_BIG_ENDIAN = 5&
    REG_LINK = 6&
    REG_MULTI_SZ = 7&
    REG_RESOURCE_LIST = 8&
    REG_FULL_RESOURCE_DESCRIPTOR = 9&
    REG_RESOURCE_REQUIREMENTS_LIST = 10&
End Enum

Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, ByRef phkResult As Long) As Long
Private Declare Function RegEnumKey Lib "advapi32.dll" Alias "RegEnumKeyA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, ByVal cbName As Long) As Long
Private Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpValueName As String, lpcbValueName As Long, ByVal lpReserved As Long, lpType As Long, lpData As Byte, lpcbData As Long) As Long

Private Const HKEY_LOCAL_MACHINE = &H80000002

Private Const KEY_QUERY_VALUE = &H1
Private Const READ_CONTROL = &H20000
Private Const KEY_READ_VALUES = READ_CONTROL And KEY_QUERY_VALUE

Private Const ERROR_SUCCESS = 0&
Private Const ERROR_NO_MORE_ITEMS = 259&

Private Const MAX_KEY_SIZE = 260
Private Const MAX_VALUE_SIZE = 4096
 
Private Const ctrlSet As String = "SYSTEM\CurrentControlSet\Enum\"

Public Function GetUsbDetails(ByVal s As String) As String
   Dim sKey As String
   sKey = Replace(s, "\\?\", "")
   Dim nPos As Long
   nPos = InStr(1, sKey, "{")
   If nPos > 1 Then
      sKey = Left(sKey, nPos - 1)
   End If
   sKey = ctrlSet & Replace(sKey, "#", "\")
   If Right(sKey, 1) = "\" Then sKey = Left(sKey, Len(sKey) - 1)
   GetUsbDetails = GetKeyValues(sKey)
End Function

Private Function GetKeyValues(ByVal sKey As String) As String
   Dim idx As Long, hKey As Long, ret As Long
   If RegOpenKeyEx(HKEY_LOCAL_MACHINE, sKey, 0, KEY_QUERY_VALUE, hKey) Then Exit Function
   Dim abData() As Byte
   Dim ValueName As String, ValueValue As String
   Dim lType As Long, cbDataSize As Long, sOut As String
   Do
       ValueName = String(MAX_KEY_SIZE, 0)
       cbDataSize = MAX_VALUE_SIZE
       ReDim abData(cbDataSize - 1)
       ret = RegEnumValue(hKey, idx, ValueName, MAX_KEY_SIZE, ByVal 0&, lType, abData(0), cbDataSize)
       If ret <> ERROR_SUCCESS Then
          If ret <> ERROR_NO_MORE_ITEMS Then Debug.Print ret
          Exit Do
       End If
       If cbDataSize < 1 Then cbDataSize = 1
       ReDim Preserve abData(cbDataSize - 1)
       ValueName = TrimNull(ValueName)
       ValueValue = TrimNull(GetRegData(lType, abData))
       If sOut <> "" Then sOut = sOut & vbCrLf
       sOut = sOut & ValueName & ": " & ValueValue
       idx = idx + 1
   Loop
   RegCloseKey hKey
   GetKeyValues = sOut
End Function

Private Function GetRegData(ByVal lType As RegValueTypes, abData() As Byte) As String
   Dim lData As Long, i As Long
   Dim sTemp As String
   sTemp = ""
   Select Case lType
        Case REG_SZ
             GetRegData = TrimNull(StrConv(abData, vbUnicode))
        Case REG_MULTI_SZ
              sTemp = TrimDblNull(StrConv(abData, vbUnicode))
             If sTemp = Chr(0) Then Exit Function
             GetRegData = Replace(sTemp, Chr(0), "; ")
             'GetRegData = Replace(sTemp, Chr(0), vbCrLf)
        Case REG_EXPAND_SZ
             'GetRegData = ExpandEnvironmentString(TrimNull(StrConv(abData, vbUnicode)))
        Case REG_DWORD
             CopyMemory lData, abData(0), 4
             GetRegData = "0x" & Hex(lData) & "(" & lData & ")"
        Case REG_BINARY, REG_NONE
             For i = 0 To UBound(abData)
                 sTemp = sTemp & Right("00" & Hex(abData(i)), 2) & " "
             Next i
             GetRegData = Left(sTemp, Len(sTemp) - 1)
        Case Else
             GetRegData = "Unsupported data type (length=" & UBound(abData) + 1 & ")"
   End Select
End Function

Public Function TrimDblNull(startstr As String) As String
   Dim pos As Integer
   pos = InStr(startstr, Chr$(0) & Chr$(0))
   If pos Then
      TrimDblNull = Left$(startstr, pos - 1)
      Exit Function
   End If
   TrimDblNull = startstr
End Function

